unit TDBConnection_U;

interface

uses
  DB, ADODB, Classes, Forms, DBGrids, SysUtils, Dialogs, Windows;

type
  TDBConnection = class(TObject)
    private
      var
        connection: TADOConnection;
        sourceA, sourceB, srcSQL: TDataSource;
        query: TADOQuery;
    public
      procedure DBConnectionConfig(owner: TComponent);
      procedure DBConnect(var tableA, tableB: TADOTable);
      procedure DBDisconnect(var tableA, tableB: TADOTable);
      procedure DBTableConfig(var tableA, tableB: TADOTable);
      procedure DBQueryConfig();

      procedure DBGridConfig(var gridA, gridB, gridSQL: TDBGrid);
      procedure UpdateGridA(var gridA: TDBGrid);
      procedure UpdateGridB(var gridB: TDBGrid);
      procedure UpdateGridSQL(var gridSQL: TDBGrid);

      //  Method to call for data EXTRACTION (SELECT statements)...
      procedure DBQuery(sql: string; var gridSQL: TDBGrid);
      //  Method to call for data MODIFICATION (INSERT, UPDATE, DELETE statements)...
      procedure DBModify(sql: string);

      procedure DBRestore(var tableA, tableB: TADOTable);
  end;

implementation

{ TDBConnection }

const
  DB_IDENTIFIER = 'DB.mdb';
  TABLE_A = 'tblDrivers';
  TABLE_B = 'tblCars';

procedure TDBConnection.DBConnect(var tableA, tableB: TADOTable);
begin
  connection.Connected := True;
  tableA.Open();
  tableB.Open();
end;

procedure TDBConnection.DBConnectionConfig(owner: TComponent);
begin
  connection := TADOConnection.Create(owner);
  connection.ConnectionString := 'Provider=Microsoft.Jet.OLEDB.4.0;Data Source=' + DB_IDENTIFIER + ';' +
                                 'Mode=ReadWrite;Persist Security Info=False;Jet OLEDB:Database Password=Java963';
  connection.LoginPrompt := False;
end;

procedure TDBConnection.DBDisconnect(var tableA, tableB: TADOTable);
begin
  connection.Connected := False;
  tableA.Close();
  tableB.Close();
  query.Close();
end;

procedure TDBConnection.DBGridConfig(var gridA, gridB, gridSQL: TDBGrid);
begin
  gridA.DataSource := sourceA;
  gridB.DataSource := sourceB;
  gridSQL.DataSource := srcSQL;
end;

procedure TDBConnection.DBModify(sql: string);
begin
  if Length(sql) = 0 then
    begin
      ShowMessage('No SQL code has been entered.');
    end
  else
    begin
      try
        query.Close();
        query.SQL.Text := sql;
        query.ExecSQL();

        ShowMessage('Record(s) affected: ' + IntToStr(query.RowsAffected));
      except
        on e: Exception do
          begin
            ShowMessage(e.Message);
          end;
      end;
    end;
end;

procedure TDBConnection.DBQuery(sql: string; var gridSQL: TDBGrid);
begin
  if Length(sql) = 0 then
    begin
      ShowMessage('No SQL code has been entered.');
    end
  else
    begin
      try
        query.Close();
        query.SQL.Text := sql;
        query.Open();

        //  Updates the DBGrid that displays the SQL results.
        UpdateGridSQL(gridSQL);
      except
        on e: Exception do
          begin
            ShowMessage(e.Message);
          end;
      end;
    end;
end;

procedure TDBConnection.DBQueryConfig();
begin
  //  AOwner param set to nil for backwards compatibility.
  query := TADOQuery.Create(nil);
  query.Connection := connection;

  srcSQL := TDataSource.Create(nil);
  srcSQL.DataSet := query;
end;

procedure TDBConnection.DBRestore(var tableA, tableB: TADOTable);
begin
  DBDisconnect(tableA, tableB);

  if DeleteFile('DB.mdb') and CopyFile('DBBackup.mdb', 'DB.mdb', False) then
    begin
      ShowMessage('Database restored!');
    end;

  DBConnect(tableA, tableB);
end;

procedure TDBConnection.DBTableConfig(var tableA, tableB: TADOTable);
begin
  //  AOwner param set to nil for backwards compatability.
  tableA := TADOTable.Create(nil);
  tableA.Connection := connection;
  tableA.TableName := TABLE_A;
  tableA.Open();

  tableB := TADOTable.Create(nil);
  tableB.Connection := connection;
  tableB.TableName := TABLE_B;
  tableB.Open();

  sourceA := TDataSource.Create(nil);
  sourceA.DataSet := tableA;

  sourceB := TDataSource.Create(nil);
  sourceB.DataSet := tableB;
end;

procedure TDBConnection.UpdateGridA(var gridA: TDBGrid);
  var
    i: Integer;
begin
  //  For display purposes: modify to adjust column widths.
  for i := 0 to gridA.Columns.Count - 1 do
    begin
      gridA.Columns[i].Width := 160;
    end;
  gridA.Width := gridA.Columns.Count * 120 + 180;
end;

procedure TDBConnection.UpdateGridB(var gridB: TDBGrid);
  var
    i: Integer;
begin
  //  For display purposes: modify to adjust column widths.
  gridB.Columns[0].Width := 120;
  for i := 1 to gridB.Columns.Count - 2 do
    begin
      gridB.Columns[i].Width := 120;
    end;
  gridB.Columns[gridB.Columns.Count - 1].Width := 100;
  gridB.Width := gridB.Columns.Count * 120 ;
end;

procedure TDBConnection.UpdateGridSQL(var gridSQL: TDBGrid);
  var
    i: Integer;
begin
  //  For display purposes: modify to adjust column widths.
  for i := 0 to gridSQL.Columns.Count - 1 do
    begin
      gridSQL.Columns[i].Width := 160;
    end;
  gridSQL.Width := gridSQL.Columns.Count * 160 + 40;
end;

end.
